/*
 * Copyright (C) 2015 PHYTEC Messtechnik GmbH,
 * Author:   Alexander Fratzer <a.fratzer@phytec.de>
 *           Alexander Aring   <a.aring@phytec.de>
 * Released under the MIT license
 */

#include "regparse.h"
#include "errhandl.h"

/*-----------------------------------------------------------------------------
 *  Trim a string from the right side.
 *-----------------------------------------------------------------------------*/
static char *rtrim(char *str)
{
	char *ptr;
	int len;

	len = strlen(str);

	for (ptr = str + len - 1; ptr >= str && isspace((int)*ptr); --ptr) ;

	ptr[1] = '\0';

	return str;
}

/*-----------------------------------------------------------------------------
 *  Trim a string from the left side. 
 *-----------------------------------------------------------------------------*/
static char *ltrim(char *str)
{
	char *ptr;
	int len;

	for (ptr = str; *ptr && isspace((int)*ptr); ++ptr) ;

	len = strlen(ptr);
	memmove(str, ptr, len + 1);

	return str;
}

/*-----------------------------------------------------------------------------
 *  Trim a string from both sides.
 *-----------------------------------------------------------------------------*/
static char *trim(char *str)
{
	char *ptr;
	ptr = rtrim(str);
	str = ltrim(ptr);
	return str;
}

/*-----------------------------------------------------------------------------
 *  Get string contents until character #.
 *-----------------------------------------------------------------------------*/
static void remove_comment(char *line)
{
	char *str;

	str = strchr(line, '#');
	if (str == NULL)
		return;

	*str = '\0';
}

/*-----------------------------------------------------------------------------
 *  Convert a line to for call-by reference values.
 *-----------------------------------------------------------------------------*/
static int
line_conv(char *line, __u8 * addr, __u8 * reg, __u16 * val, char *mode)
{
	char *str;

	/* Address value */
	str = strtok(line, ",");
	*addr = strtoul(str, NULL, 16);
	if (*addr == ULONG_MAX) {
		perror("strtoul");
		return -1;
	}

	/* Register value */
	str = strtok(NULL, ",");
	*reg = strtoul(str, NULL, 16);
	if (*reg == ULONG_MAX) {
		perror("strtoul");
		return -1;
	}

	/* Data value */
	str = strtok(NULL, ",");
	*val = strtoul(str, NULL, 0);
	if (*val == ULONG_MAX) {
		perror("strtoul");
		return -1;
	}

	/* Optional mode string */
	str = strtok(NULL, ",");
	if (str != NULL)
		*mode = *str;
	else
		*mode = '\0';

	return 0;
}

/*-----------------------------------------------------------------------------
 *  Do a file register file parse.
 *  For each register command exectue do_something.
 *-----------------------------------------------------------------------------*/
int reg_file_parse(char *filepath,
		   int do_something(uint32_t address, uint64_t reg,
				    uint64_t val, char mode,
				    uint32_t use_subdev), uint32_t use_sd)
{
	int err;
	FILE *file = NULL;
	char line[MAX_LINE_LENGTH];
	__u8 addr;
	__u8 reg;
	__u16 val;
	char mode;

	CHECK_VAL(!filepath, goto out;);

	addr = 0;
	reg = 0;
	val = 0;

	file = fopen(filepath, "r");
	if (!file) {
		perror("fopen");
		goto out;
	}

	while (fgets(line, MAX_LINE_LENGTH, file)) {
		remove_comment(line);
		trim(line);
		if (*line == '\0')
			continue;

		err = line_conv(line, &addr, &reg, &val, &mode);
		if (err != 0)
			goto close;

		err = do_something(addr, reg, val, mode, use_sd);
		if (err != 0)
			goto close;

	}

	err = fclose(file);
	if (err == EOF)
		perror("fclose");

	return 0;
 close:
	err = fclose(file);
 out:
	return -1;
}
